/*
 * [The "BSD licence"]
 * Copyright (c) 2013 Dandelion
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Dandelion nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.dandelion.webanalytics.thymeleaf.processor;

import com.github.dandelion.thymeleaf.processor.DandelionAttrProcessor;
import com.github.dandelion.thymeleaf.util.ArgumentsUtil;
import com.github.dandelion.thymeleaf.util.AttributesUtil;
import com.github.dandelion.webanalytics.core.WebAnalyticsRequestContext;
import com.github.dandelion.webanalytics.thymeleaf.dialect.DandelionWebAnalyticsDialect;
import com.github.dandelion.webanalytics.thymeleaf.dialect.WebAnalyticsAttributeName;
import org.thymeleaf.Arguments;
import org.thymeleaf.dom.Element;
import org.thymeleaf.processor.ProcessorResult;

import java.util.Map;

/**
 * Processor for all web analytics attributes ({@link WebAnalyticsAttributeName})
 */
public class WebAnalyticsAttrProcessor extends DandelionAttrProcessor {

    public WebAnalyticsAttrProcessor(String attributeName) {
        super(attributeName);
    }

    @Override
    public int getPrecedence() {
        return DandelionWebAnalyticsDialect.HIGHEST_PRECEDENCE;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ProcessorResult doProcessAttribute(Arguments arguments, Element element, String attributeName) {
        String strippedAttributeName
                = AttributesUtil.stripPrefix(attributeName, DandelionWebAnalyticsDialect.DIALECT_PREFIX);
        WebAnalyticsAttributeName webAnalyticsAttributeName
                = (WebAnalyticsAttributeName) AttributesUtil.find(strippedAttributeName, WebAnalyticsAttributeName.values());

        // store as property the value for future use
        String value = element.getAttributeValue(attributeName);
        Map<String,Object> attributes = arguments.getExecutionAttributes();
        attributes.put(webAnalyticsAttributeName.getAttribute(), value);

        // extract the needed 2 properties
        String provider = (String) attributes.get(WebAnalyticsAttributeName.PROVIDER.getAttribute());
        String token = (String) attributes.get(WebAnalyticsAttributeName.TOKEN.getAttribute());

        // we configure the asset context only if we have the 2 needed properties
        if(provider != null && token != null) {
            WebAnalyticsRequestContext
                    .get(ArgumentsUtil.getWebContext(arguments).getHttpServletRequest())
                    .setup(provider, token);
        }

        return ProcessorResult.ok();
    }
}